"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const visitor_1 = require("./visitor");
exports.plugin = (schema, documents, config) => {
    const imports = ['GraphQLResolveInfo'];
    const hasScalars = Object.values(schema.getTypeMap())
        .filter(t => t.astNode)
        .some(graphql_1.isScalarType);
    if (hasScalars) {
        imports.push('GraphQLScalarType', 'GraphQLScalarTypeConfig');
    }
    const visitor = new visitor_1.TypeScriptResolversVisitor(config, schema);
    const header = `
import { ${imports.join(', ')} } from 'graphql';

export type ResolverFn<TResult, TParent, TContext, TArgs> = (
  parent?: TParent,
  args?: TArgs,
  context?: TContext,
  info?: GraphQLResolveInfo
) => Promise<TResult> | TResult;

export type StitchingResolver<TResult, TParent, TContext, TArgs> = {
  fragment: string;
  resolve: ResolverFn<TResult, TParent, TContext, TArgs>;
};

export type Resolver<TResult, TParent = {}, TContext = {}, TArgs = {}> =
  | ResolverFn<TResult, TParent, TContext, TArgs>
  | StitchingResolver<TResult, TParent, TContext, TArgs>;

export type SubscriptionSubscribeFn<TResult, TParent, TContext, TArgs> = (
  parent?: TParent,
  args?: TArgs,
  context?: TContext,
  info?: GraphQLResolveInfo
) => AsyncIterator<TResult> | Promise<AsyncIterator<TResult>>;

export type SubscriptionResolveFn<TResult, TParent, TContext, TArgs> = (
  parent?: TParent,
  args?: TArgs,
  context?: TContext,
  info?: GraphQLResolveInfo
) => TResult | Promise<TResult>;

export interface ISubscriptionResolverObject<TResult, TParent, TContext, TArgs> {
  subscribe: SubscriptionSubscribeFn<TResult, TParent, TContext, TArgs>;
  resolve?: SubscriptionResolveFn<TResult, TParent, TContext, TArgs>;
}

export type SubscriptionResolver<TResult, TParent = {}, TContext = {}, TArgs = {}> =
  | ((...args: any[]) => ISubscriptionResolverObject<TResult, TParent, TContext, TArgs>)
  | ISubscriptionResolverObject<TResult, TParent, TContext, TArgs>;

export type TypeResolveFn<TTypes, TParent = {}, TContext = {}> = (
  parent?: TParent,
  context?: TContext,
  info?: GraphQLResolveInfo
) => Maybe<TTypes>;

export type NextResolverFn<T> = () => Promise<T>;

export type DirectiveResolverFn<TResult = {}, TParent = {}, TContext = {}, TArgs = {}> = (
  next?: NextResolverFn<TResult>,
  parent?: TParent,
  args?: TArgs,
  context?: TContext,
  info?: GraphQLResolveInfo
) => TResult | Promise<TResult>;
`;
    const printedSchema = graphql_1.printSchema(schema);
    const astNode = graphql_1.parse(printedSchema);
    const visitorResult = graphql_1.visit(astNode, { leave: visitor });
    const { getRootResolver, getAllDirectiveResolvers, mappersImports } = visitor;
    return [
        ...mappersImports,
        header,
        ...visitorResult.definitions.filter(d => typeof d === 'string'),
        getRootResolver(),
        getAllDirectiveResolvers()
    ].join('\n');
};
//# sourceMappingURL=index.js.map